#ifndef MYDEF_H
#define MYDEF_H
/*
	For WIN32
*/

#ifndef __WINDOWS_H
#include <windows.h>
#endif

// ***********************************************************************
// SAVE TYPES
// ***********************************************************************
/*
	Save Conventions:
	     types int & short changes by system 16bit or 32bit
	     to avoid differences between DB files and other objs
	     in save/load functions assumed:
	      char,u_char	8bit
	      short		directly unused
	      int		directrly unused
	      long,u_long	32 bit
	      INT,u_INT		16 bit type instead int,short

	      enum directly conversion to INT based types
	      save( (INT)enum_value );
*/

//Here int 32-bit -same as long
#ifndef Int
typedef short int Int;
#endif
#ifndef Int_p
typedef Int* Int_p;
#endif

#ifndef u_Int
typedef unsigned short int u_Int;
#endif
#ifndef u_Int_p
typedef u_Int* u_Int_p;
#endif

// ***********************************************************************
// STD TYPES
// ***********************************************************************
#ifndef char_p
typedef char* char_p;
#endif
#ifndef u_char
typedef unsigned char u_char;
#endif
#ifndef u_char_p
typedef u_char* u_char_p;
#endif

#ifndef int_p
typedef int* int_p;
#endif
#ifndef u_int
typedef unsigned int u_int;
#endif
#ifndef u_int_p
typedef u_int* u_int_p;
#endif

#ifndef long_p
typedef long *long_p;
#endif
#ifndef u_long
typedef unsigned long u_long;
#endif
#ifndef u_long_p
typedef u_long *u_long_p;
#endif

#ifndef double_p
typedef double *double_p;
#endif
#ifndef float_p
typedef float *float_p;
#endif

#ifndef void_p
typedef void *void_p;
#endif

// ***********************************************************************
// Error handl
// ***********************************************************************
#ifndef t_err

typedef	enum{
	ERR_NO=0,
	ERR_NOMEM,
	ERR_FIO,
	ERR_BOUND,
	ERR_BADPAR,
	ERR_ZPAR,
	ERR_ZDATA,
	ERR_NFOUND,
	ERR_NEXIST,
	ERR_INTERNAL,
	ERR_ALREADY,
	ERR_BUZY,
	ERR_UNDEF,
	ERR_BP,
	ERR_DENIED,
	ERR_BADVER,
	ERR_BADFORMAT
}t_err;
#endif

#ifndef	t_on_err
typedef enum{
	ONERR_HALT=0,
	ONERR_GO
}t_on_err;
#endif

// what do on error
extern	char	on_err;
//   -    TWinMain
extern	void	init_err(HWND module);
//For MustDie compatible linker
extern	void	init_err(void* module);

// ***********************************************************************
t_err 	errw32_(t_err err, char_p f, u_int l, char_p msg=0);

//with return type*
#pragma argsused
template <class T>
T*	errw32t_(t_err err, char_p f, u_int l, char_p msg, T* rtype)
{
	errw32_(err,f,l,msg);
	return rtype;
}

//error char_p description
char_p	strerr(t_err err);

#define	err(e)	    errw32_((e),__FILE__,__LINE__)
#define	err2(e,m)   errw32_((e),__FILE__,__LINE__,(m))
#define	err3(e,m,t) errw32t_((e),__FILE__,__LINE__,(m),(t))
#define warn(m)	    (err2(ERR_NO,(m)))

// ***********************************************************************
// others
// ***********************************************************************
// get line from fp: max_len bytes max (skip to EOL if line more)
t_err		getln(char_p buf, u_int max_len, FILE *fp);

// ***********************************************************************
//Tobj_on_stream
// ***********************************************************************
// /  / 
class	Toos;
typedef Toos *Toos_p;

//  /
typedef enum
{
	OBJ_LOAD=0,
	OBJ_SAVE
}t_oos_operation;

// /  
//   . 
#define CALL_Toos 	t_err oos(t_oos_operation oper,FILE *f)
#define CALL_Toos2()	t_err Toos::oos(oper,f)

//   ,   . 
//  
#define SELF_Toos	protected:\
			t_err obj_load(FILE *fi);\
			t_err obj_save(FILE *fo);

#define SELF_Toos2	protected:\
			t_err obj_load(FILE *fi){return ERR_NO;}\
			t_err obj_save(FILE *fo){return ERR_NO;}

// ***********************************************************************
// /  / 
class	Toos
{
// 
public:
	// /  
	//   . 
	virtual CALL_Toos
	 {return (oper==OBJ_LOAD)?obj_load(f):obj_save(f);}

protected:
	//   ,   . 
	//  
	virtual t_err obj_load(FILE *fi)=0;
	virtual t_err obj_save(FILE *fo)=0;
};

// ***********************************************************************
//蠡 min
#if 1
template <class T> T& my_min(T& a, T& b)
{
	return (a<b)? a:b;
}

template <class T> T& my_max(T& a, T& b)
{
	return (a>b)? a:b;
}

#define my_abs(a) ((a<0)? -a: a)

#endif

// ***********************************************************************
#endif
