#include <stdio.h>
#include <stdlib.h>

#include <mydef.h>
#include "tstring.h"

// ***********************************************************************
//#define  TSTRING_DEBUG

// ***********************************************************************
//   -  
	 Tstring::Tstring( char_p src, u_int maxlen )
{
u_int		str_len=0;

#ifdef  TSTRING_DEBUG
	printf("Tstring::Tstring(char_p, u_int): %s %u [%p]\n",src,maxlen,this);
#endif

	mem_size=0;
	data_p=NULL;
	compare_equ_min_size=compare_less_min_size=0;

	if(src)mem_size=str_len=strlen(src);
	if(maxlen>mem_size)mem_size=maxlen;
	if(!mem_size)return;

	data_p=new char[mem_size+8];
	if(!data_p){ err(ERR_NOMEM);return; }

	if(str_len)strcpy(data_p,src);
	else *data_p=0;
}

// ***********************************************************************
//       
	 Tstring::Tstring( Tstring& src )
{
#ifdef  TSTRING_DEBUG
	printf("Tstring::Tstring(Tstring&): [%p]\n",this);
#endif
	compare_equ_min_size=src.compare_equ_min_size;
	compare_less_min_size=src.compare_less_min_size;

	mem_size=src.mem_size;
	data_p=NULL;
	if(!mem_size)return;

	data_p=new char [mem_size+8];
	if(!data_p){ err(ERR_NOMEM);return; }

	memcpy(data_p,src.data_p,mem_size+1);
}

// ***********************************************************************
	 Tstring::~Tstring()
{
#ifdef  TSTRING_DEBUG
	printf("Tstring::~Tstring() [%p]\n",this);
#endif
	if(data_p)delete data_p;
}

// ***********************************************************************
// 
// ***********************************************************************
//  , / . . 
#pragma argsused
t_err	Tstring::obj_load(FILE *fi)
{
u_Int		len=0;

	if(!fi)return err(ERR_ZPAR);
	~(*this);

	fread(&len,1,sizeof(len),fi);
	if(!len)return ERR_NO;

	mem_size=len;
	data_p=new char[mem_size+8];
	if(!data_p){ return err(ERR_NOMEM); }

	fread(data_p,1,len,fi);
	data_p[len]=0;

	return ERR_NO;
}

// ***********************************************************************
//  , / . . 
#pragma argsused
t_err   Tstring::obj_save(FILE *fo)
{
u_Int		len=0;

	if(!fo)return err(ERR_ZPAR);

	if(data_p)len=(u_Int)strlen(data_p);

	fwrite(&len,1,sizeof(len),fo);
	if(len)fwrite(data_p,1,len,fo);

	return ERR_NO;
}

// ***********************************************************************
// 
// ***********************************************************************
char 	Tstring::operator ==( Tstring& str )
{
u_int		mi,s,d;

	// 0
	if((!data_p[0])&&(!str.data_p[0]))return 1;
	s=len();
	d=str.len();

	if(!compare_equ_min_size)
	  return char( (s!=d)? 0: !strcmp( data_p, str.data_p ) );

	//  
	mi=my_min(s,d);
	if(!mi)return 0;

	return (char) !memcmp( data_p, str.data_p, mi );
}

// ***********************************************************************
char		 Tstring::operator <( Tstring & str )
{
	return compare( str, Tstring_compare_less);
}

// ***********************************************************************
char		Tstring::operator >( Tstring & str )
{
	return compare( str, Tstring_compare_more);
}

// ***********************************************************************
//   < >
/*
char		Tstring::compare ( Tstring & str,
				     char  (* cmpu)(u_int,u_int),
				      char  (* cmpi)(int,int)
				 )
*/
char		Tstring::compare ( Tstring & str,t_tstring_compare tstring_compare )
{
u_int		mi,s,d;

	// 0
	if((!data_p[0])&&(!str.data_p[0]))return 1;
	s=len();
	d=str.len();

	if(!compare_less_min_size)
	 {
	  if(s!=d)return char( (tstring_compare==Tstring_compare_less)? (s<d): (s>d) );
	  return char( (tstring_compare==Tstring_compare_less)?
		 (strcmp(data_p,str.data_p)<0):
		 (strcmp(data_p,str.data_p)>0) );
	 }

	//  
	mi=my_min(s,d);
	//?   -  
	if(!mi)return char( (tstring_compare==Tstring_compare_less)? (s<d): (s>d) );

	return	char( (tstring_compare==Tstring_compare_less)?
		(memcmp(data_p,str.data_p,mi)<0):
		(memcmp(data_p,str.data_p,mi)>0) );
}

// ***********************************************************************
//
// ***********************************************************************
Tstring & 	 Tstring::operator= ( Tstring & src )
{
#ifdef  TSTRING_DEBUG
	printf("Tstring::operator=(Tstring&): [%p]\n",this);
#endif
	/*
	compare_equ_min_size=src.compare_equ_min_size;
	compare_less_min_size=src.compare_less_min_size;
	*/

	if(data_p)delete data_p;
	data_p=NULL;
	mem_size=src.mem_size;
	if(!mem_size)return *this;

	data_p=new char [mem_size+8];
	if(!data_p){ err(ERR_NOMEM);return *this; }

	memcpy(data_p,src.data_p,mem_size+1);
	return *this;
}

// ***********************************************************************
Tstring & 	 Tstring::operator +=( Tstring & src )
{
u_int		dst_len;
char_p		p;

#ifdef  TSTRING_DEBUG
	printf("Tstring::operator+=(Tstring&): [%p]\n",this);
#endif

	if(!src.data_p)return *this;
	dst_len=src.len();
	if(data_p)dst_len+=len();

	if(mem_size<dst_len)
	 {
	  mem_size=dst_len;
	  p=new char[mem_size+8];
	  if(!p){ err(ERR_NOMEM);return *this; }

	  if(data_p)
	   {
	    strcpy(p,data_p);
	    delete data_p;
	   }
	  else *p=0;

	  data_p=p;
	 }
	strcpy(data_p+strlen(data_p),src.data_p);
	return *this;
}

// ***********************************************************************
Tstring 	 Tstring::operator +( Tstring & src )
{
Tstring		res(*this);

	//printf("%s\n",-res);
	if(src.data_p)res+=src;
	//printf("%s\n",-res);
	return res;
}

// ***********************************************************************
//  char_p
// ***********************************************************************
//operator char_p();	-  printf  string.h
char_p		Tstring::operator -()
{
	return data_p;
}

// ***********************************************************************
Tstring&	 Tstring::operator =( char_p src )
{
#ifdef  TSTRING_DEBUG
	printf("Tstring::operator=(char_p): %s [%p]\n",src,this);
#endif

	if(data_p)delete data_p;
	data_p=NULL;
	mem_size=0;
	if(!src)return *this;
	mem_size=strlen(src);
	if(!mem_size)return *this;

	data_p=new char [mem_size+8];
	if(!data_p){ err(ERR_NOMEM);return *this; }

	strcpy(data_p,src);
	return *this;
}

// ***********************************************************************
Tstring& 	 Tstring::operator +=( char_p src )
{
u_int		dst_len;
char_p		p;

#ifdef  TSTRING_DEBUG
	printf("Tstring::operator+=(char_p): %s [%p]\n",src,this);
#endif

	if(!src)return *this;
	dst_len=strlen(src);
	if(data_p)dst_len+=strlen(data_p);

	if(mem_size<dst_len)
	 {
	  mem_size=dst_len;
	  p=new char[mem_size+8];
	  if(!p){ err(ERR_NOMEM);return *this; }

	  if(data_p)
	   {
	    strcpy(p,data_p);
	    delete data_p;
	   }
	  else *p=0;

	  data_p=p;
	 }
	strcpy(data_p+strlen(data_p),src);
	return *this;
}

// ***********************************************************************
// 
// ***********************************************************************
//     :      
void		Tstring::compact()
{
u_int		rq_len;
char_p		p;

	if(!data_p)return;
	rq_len=strlen(data_p);
	if(mem_size<=rq_len+8)return;

	mem_size=rq_len;
	p=new char[mem_size+8];
	if(!p){ err(ERR_NOMEM);return; }

	memcpy(p,data_p,mem_size+1);
	delete data_p;
	data_p=p;
}
// ***********************************************************************
//   
Tstring& 	 Tstring::operator ~()
{
#ifdef  TSTRING_DEBUG
	printf("Tstring::operator~(): [%p]\n",this);
#endif

	if(data_p)delete data_p;
	data_p=NULL;
	mem_size=0;

	return *this;
}
// ***********************************************************************
